@@ -298,7 +298,7 @@ mysql_deparse_target_list(StringInfo buf,
 	*retrieved_attrs = NIL;
 	for (i = 1; i <= tupdesc->natts; i++)
 	{
-		Form_pg_attribute attr = tupdesc->attrs[i - 1];
+		Form_pg_attribute attr = TupleDescAttr(tupdesc, i - 1);
 
 		/* Ignore dropped attributes. */
 		if (attr->attisdropped)
@@ -415,7 +415,11 @@ mysql_deparse_column_ref(StringInfo buf, int varno, int varattno, PlannerInfo *r
 	 * option, use attribute name.
 	 */
 	if (colname == NULL)
+#if PG_VERSION_NUM >= 110000
+		colname = get_attname(rte->relid, varattno, false);
+#else
 		colname = get_relid_attribute_name(rte->relid, varattno);
+#endif
 
 	appendStringInfoString(buf, mysql_quote_identifier(colname, '`'));
 }
