@@ -1,6 +1,6 @@
 /*-------------------------------------------------------------------------
  *
- * mysql_fdw--1.0.sql
+ * mysql_fdw--1.1.sql
  * 			Foreign-data wrapper for remote MySQL servers
  *
  * Portions Copyright (c) 2012-2014, PostgreSQL Global Development Group
@@ -8,7 +8,7 @@
  * Portions Copyright (c) 2004-2014, EnterpriseDB Corporation.
  *
  * IDENTIFICATION
- * 			mysql_fdw--1.0.sql
+ * 			mysql_fdw--1.1.sql
  *
  *-------------------------------------------------------------------------
  */
@@ -27,3 +27,7 @@ LANGUAGE C STRICT;
 CREATE FOREIGN DATA WRAPPER mysql_fdw
   HANDLER mysql_fdw_handler
   VALIDATOR mysql_fdw_validator;
+
+CREATE OR REPLACE FUNCTION mysql_fdw_version()
+  RETURNS pg_catalog.int4 STRICT
+  AS 'MODULE_PATHNAME' LANGUAGE C;
