@@ -82,9 +82,9 @@
 
 /*
  * In PG 9.5.1 the number will be 90501,
- * our version is 2.4.0 so number will be 20400
+ * our version is 2.5.1 so number will be 20501
  */
-#define CODE_VERSION   20400
+#define CODE_VERSION   20501
 
 PG_MODULE_MAGIC;
 
@@ -200,7 +200,7 @@ static int interactive_timeout = INTERACTIVE_TIMEOUT;
 bool
 mysql_load_library(void)
 {
-#if defined(__APPLE__)
+#if defined(__APPLE__) || defined(__FreeBSD__)
 	/*
 	 * Mac OS/BSD does not support RTLD_DEEPBIND, but it still
 	 * works without the RTLD_DEEPBIND
@@ -426,9 +426,13 @@ mysqlBeginForeignScan(ForeignScanState *node, int eflags)
 
 	festate->temp_cxt = AllocSetContextCreate(estate->es_query_cxt,
 											  "mysql_fdw temporary data",
+#if PG_VERSION_NUM >= 110000
+											  ALLOCSET_DEFAULT_SIZES);
+#else
 											  ALLOCSET_SMALL_MINSIZE,
 											  ALLOCSET_SMALL_INITSIZE,
 											  ALLOCSET_SMALL_MAXSIZE);
+#endif
 
 	if (wait_timeout > 0)
 	{
@@ -534,10 +538,10 @@ mysqlBeginForeignScan(ForeignScanState *node, int eflags)
 	foreach(lc, festate->retrieved_attrs)
 	{
 		int attnum = lfirst_int(lc) - 1;
-		Oid pgtype = tupleDescriptor->attrs[attnum]->atttypid;
-		int32 pgtypmod = tupleDescriptor->attrs[attnum]->atttypmod;
+		Oid pgtype = TupleDescAttr(tupleDescriptor, attnum)->atttypid;
+		int32 pgtypmod = TupleDescAttr(tupleDescriptor, attnum)->atttypmod;
 
-		if (tupleDescriptor->attrs[attnum]->attisdropped)
+		if (TupleDescAttr(tupleDescriptor, attnum)->attisdropped)
 			continue;
 
 		festate->table->column[atindex]._mysql_bind = &festate->table->_mysql_bind[atindex];
@@ -640,8 +644,8 @@ mysqlIterateForeignScan(ForeignScanState *node)
 		foreach(lc, festate->retrieved_attrs)
 		{
 			int attnum = lfirst_int(lc) - 1;
-			Oid pgtype = tupleDescriptor->attrs[attnum]->atttypid;
-			int32 pgtypmod = tupleDescriptor->attrs[attnum]->atttypmod;
+			Oid pgtype = TupleDescAttr(tupleDescriptor, attnum)->atttypid;
+			int32 pgtypmod = TupleDescAttr(tupleDescriptor, attnum)->atttypmod;
 
 			tupleSlot->tts_isnull[attnum] = festate->table->column[attid].is_null;
 			if (!festate->table->column[attid].is_null)
@@ -699,10 +703,17 @@ mysqlExplainForeignScan(ForeignScanState *node, ExplainState *es)
 	if (es->verbose)
 	{
 		if (strcmp(options->svr_address, "127.0.0.1") == 0 || strcmp(options->svr_address, "localhost") == 0)
+#if PG_VERSION_NUM >= 110000
+			ExplainPropertyInteger("Local server startup cost", NULL, 10, es);
+#else
 			ExplainPropertyLong("Local server startup cost", 10, es);
+#endif
 		else
+#if PG_VERSION_NUM >= 110000
+			ExplainPropertyInteger("Remote server startup cost", NULL, 25, es);
+#else
 			ExplainPropertyLong("Remote server startup cost", 25, es);
-
+#endif
 		ExplainPropertyText("Remote query", festate->query, es);
 	}
 }
@@ -1286,7 +1297,7 @@ mysqlPlanForeignModify(PlannerInfo *root,
 
 		for (attnum = 1; attnum <= tupdesc->natts; attnum++)
 		{
-			Form_pg_attribute attr = tupdesc->attrs[attnum - 1];
+			Form_pg_attribute attr = TupleDescAttr(tupdesc, attnum - 1);
 
 			if (!attr->attisdropped)
 				targetAttrs = lappend_int(targetAttrs, attnum);
@@ -1322,7 +1333,11 @@ mysqlPlanForeignModify(PlannerInfo *root,
 		targetAttrs = lcons_int(1, targetAttrs);
 	}
 
+#if PG_VERSION_NUM >= 110000
+	attname = get_attname(foreignTableId, 1, false);
+#else
 	attname = get_relid_attribute_name(foreignTableId, 1);
+#endif
 
 	/*
 	 * Construct the SQL command string.
@@ -1407,15 +1422,19 @@ mysqlBeginForeignModify(ModifyTableState *mtstate,
 	fmstate->p_nums = 0;
 	fmstate->temp_cxt = AllocSetContextCreate(estate->es_query_cxt,
 											  "mysql_fdw temporary data",
+#if PG_VERSION_NUM >= 110000
+											  ALLOCSET_DEFAULT_SIZES);
+#else
 											  ALLOCSET_SMALL_MINSIZE,
 											  ALLOCSET_SMALL_INITSIZE,
 											  ALLOCSET_SMALL_MAXSIZE);
+#endif
 
 	/* Set up for remaining transmittable parameters */
 	foreach(lc, fmstate->retrieved_attrs)
 	{
 		int attnum = lfirst_int(lc);
-		Form_pg_attribute attr = RelationGetDescr(rel)->attrs[attnum - 1];
+		Form_pg_attribute attr = TupleDescAttr(RelationGetDescr(rel), attnum - 1);
 
 		Assert(!attr->attisdropped);
 
@@ -1503,7 +1522,7 @@ mysqlExecForeignInsert(EState *estate,
 		int attnum = lfirst_int(lc) - 1;
 
 		bool *isnull = (bool*) palloc0(sizeof(bool) * n_params);
-		Oid type = slot->tts_tupleDescriptor->attrs[attnum]->atttypid;
+		Oid type = TupleDescAttr(slot->tts_tupleDescriptor, attnum)->atttypid;
 
 		value = slot_getattr(slot, attnum + 1, &isnull[attnum]);
 
@@ -1611,7 +1630,7 @@ mysqlExecForeignUpdate(EState *estate,
 		if (attnum == 1)
 			continue;
 
-		type = slot->tts_tupleDescriptor->attrs[attnum - 1]->atttypid;
+		type = TupleDescAttr(slot->tts_tupleDescriptor, attnum - 1)->atttypid;
 		value = slot_getattr(slot, attnum, (bool*)(&isnull[i]));
 
 		mysql_bind_sql_var(type, bindnum, value, mysql_bind_buffer, &isnull[i]);
@@ -1688,7 +1707,7 @@ mysqlAddForeignUpdateTargets(Query *parsetree,
 	 * What we need is the rowid which is the first column
 	 */
 	Form_pg_attribute attr =
-	RelationGetDescr(target_relation)->attrs[0];
+				TupleDescAttr(RelationGetDescr(target_relation), 0);
 
 	/* Make a Var representing the desired value */
 	var = makeVar(parsetree->resultRelation,
@@ -1908,8 +1927,10 @@ mysqlImportForeignSchema(ImportForeignSchemaStmt *stmt, Oid serverOid)
                      "    WHEN c.DATA_TYPE = 'datetime' THEN 'timestamp'"
                      "    WHEN c.DATA_TYPE = 'longtext' THEN 'text'"
                      "    WHEN c.DATA_TYPE = 'mediumtext' THEN 'text'"
+                     "    WHEN c.DATA_TYPE = 'tinytext' THEN 'text'"
                      "    WHEN c.DATA_TYPE = 'blob' THEN 'bytea'"
                      "    WHEN c.DATA_TYPE = 'mediumblob' THEN 'bytea'"
+                     "    WHEN c.DATA_TYPE = 'longblob' THEN 'bytea'"
                      "    ELSE c.DATA_TYPE"
                      "  END,"
                      "  c.COLUMN_TYPE,"
@@ -2097,7 +2118,7 @@ prepare_query_params(PlanState *node,
 		Oid			typefnoid;
 		bool		isvarlena;
 
-		*param_types[i] = exprType(param_expr);
+		(*param_types)[i] = exprType(param_expr);
 
 		getTypeOutputInfo(exprType(param_expr), &typefnoid, &isvarlena);
 		fmgr_info(typefnoid, &(*param_flinfo)[i]);
@@ -2112,7 +2133,11 @@ prepare_query_params(PlanState *node,
 	 * benefit, and it'd require postgres_fdw to know more than is desirable
 	 * about Param evaluation.)
 	 */
+#if PG_VERSION_NUM >= 100000
+	*param_exprs = ExecInitExprList(fdw_exprs, node);
+#else
 	*param_exprs = (List *) ExecInitExpr((Expr *) fdw_exprs, node);
+#endif
 
 	/* Allocate buffer for text form of query parameters. */
 	*param_values = (const char **) palloc0(numParams * sizeof(char *));
